@@ -52,9 +52,7 @@ def connect_button(button, func):
 
 def connect_checkbox(widget, func):
     """Connect a checkbox to a function taking bool"""
-    widget.clicked.connect(
-        lambda *args, **kwargs: func(get(checkbox)), type=Qt.QueuedConnection
-    )
+    widget.clicked.connect(lambda value: func(value), type=Qt.QueuedConnection)
 
 
 def connect_released(button, func):
@@ -208,9 +206,9 @@ def splitter(orientation, *widgets):
     return layout
 
 
-def label(text=None, align=None, fmt=None, selectable=True):
+def label(text=None, align=None, fmt=None, selectable=True, tooltip=None, parent=None):
     """Create a QLabel with the specified properties"""
-    widget = QtWidgets.QLabel()
+    widget = QtWidgets.QLabel(parent)
     if align is not None:
         widget.setAlignment(align)
     if fmt is not None:
@@ -220,6 +218,32 @@ def label(text=None, align=None, fmt=None, selectable=True):
         widget.setOpenExternalLinks(True)
     if text:
         widget.setText(text)
+    if tooltip:
+        widget.setToolTip(tooltip)
+    return widget
+
+
+def plain_text_label(text=None, align=None, selectable=True, tooltip=None, parent=None):
+    """Create a QLabel that display plain text"""
+    return label(
+        text=text,
+        align=align,
+        fmt=Qt.PlainText,
+        selectable=selectable,
+        tooltip=tooltip,
+        parent=parent,
+    )
+
+
+def pixmap_label(icon, width, height=None, tooltip=None, parent=None):
+    """Create a QLabel that displays a pixmap"""
+    if height is None:
+        height = width
+    widget = QtWidgets.QLabel(parent)
+    pixmap = icon.pixmap(width, height)
+    widget.setPixmap(pixmap)
+    if tooltip:
+        widget.setToolTip(tooltip)
     return widget
 
 
@@ -239,14 +263,55 @@ class ComboBox(QtWidgets.QComboBox):
         idx = utils.clamp(idx, 0, self.count() - 1)
         self.setCurrentIndex(idx)
 
-    def add_item(self, text, data):
+    def set_index_if_enabled(self, idx):
+        """Set the current index while ignoring disabled items"""
+        idx = utils.clamp(idx, 0, self.count() - 1)
+        model = self.model()
+        if model is None:
+            return
+        item = model.item(idx)
+        if item is None:
+            return
+        if item.isEnabled():
+            self.setCurrentIndex(idx)
+
+    def current_index(self):
+        """Return the index of the currently selected item"""
+        return self.currentIndex()
+
+    def set_item_enabled(self, idx, enabled):
+        """Enable an item on this combobox"""
+        model = self.model()
+        if model is None:
+            return
+        item = model.item(idx)
+        if item is None:
+            return
+        item.setEnabled(enabled)
+        # If the current item became disabled then select the first non-disabled item.
+        if not enabled and idx == self.currentIndex():
+            for item_idx in range(len(self.item_data)):
+                item = model.item(item_idx)
+                if item is not None and item.isEnabled():
+                    self.setCurrentIndex(item_idx)
+                    break
+
+    def add_item(self, text, data=None):
+        if data is None:
+            data = text
         self.addItem(text)
         self.item_data.append(data)
 
-    def current_data(self):
+    def value(self):
+        """Return the current data"""
         return self.item_data[self.currentIndex()]
 
+    def values(self):
+        """Return all of the valid values"""
+        return self.item_data
+
     def set_value(self, value):
+        """Set the value to one of the known values"""
         if self.transform:
             value = self.transform(value)
         try:
@@ -255,6 +320,14 @@ class ComboBox(QtWidgets.QComboBox):
             index = 0
         self.setCurrentIndex(index)
 
+    def set_current_value(self, value):
+        """Set the current value irrespective of the preset values"""
+        self.setCurrentText(value)
+
+    def current_value(self):
+        """Return the current value irrespective of the preset values"""
+        return self.currentText()
+
 
 def combo(items, editable=False, tooltip='', parent=None):
     """Create a readonly (by default) combo box from a list of items"""
@@ -268,7 +341,7 @@ def combo_mapped(data, editable=False, transform=None, parent=None):
     """Create a readonly (by default) combo box from a list of items"""
     widget = ComboBox(editable=editable, transform=transform, parent=parent)
     for k, v in data:
-        widget.add_item(k, v)
+        widget.add_item(k, data=v)
     return widget
 
 
@@ -516,8 +589,8 @@ def save_as(filename, title='Save As...'):
     return result[0]
 
 
-def existing_file(directory, title='Append...'):
-    """Creates a Save File dialog and returns a filename."""
+def existing_file(directory, title='Open File...'):
+    """Creates an OpenSave File dialog and returns a filename."""
     result = compat.getopenfilename(
         parent=active_window(), caption=title, basedir=directory
     )
@@ -966,11 +1039,18 @@ def add_menu(title, parent):
     return menu
 
 
-def create_toolbutton(text=None, layout=None, tooltip=None, icon=None, repeat=False):
+def create_toolbutton(
+    text=None,
+    layout=None,
+    tooltip=None,
+    icon=None,
+    icon_size=defs.default_icon,
+    repeat=False,
+):
     button = tool_button()
     if icon is not None:
         button.setIcon(icon)
-        button.setIconSize(QtCore.QSize(defs.default_icon, defs.default_icon))
+        button.setIconSize(QtCore.QSize(icon_size, icon_size))
     if text is not None:
         button.setText(' ' + text)
         button.setToolButtonStyle(Qt.ToolButtonTextBesideIcon)
@@ -1156,6 +1236,11 @@ class RunTask(QtCore.QObject):
         """Wait until all tasks have finished processing"""
         self.threadpool.waitForDone()
 
+    def run(self, fn, *args, **kwargs):
+        """Run a task in the background"""
+        task = SimpleTask(fn, *args, **kwargs)
+        self.start(task)
+
 
 # Syntax highlighting
 
