@@ -67,14 +67,10 @@ class AboutView(QtWidgets.QDialog):
         self.setWindowModality(Qt.WindowModal)
 
         # Top-most large icon
-        logo_pixmap = icons.cola().pixmap(defs.huge_icon, defs.large_icon)
-
-        self.logo_label = QtWidgets.QLabel()
-        self.logo_label.setPixmap(logo_pixmap)
+        self.logo_label = qtutils.pixmap_label(icons.cola(), defs.huge_icon)
         self.logo_label.setAlignment(Qt.AlignCenter)
 
-        self.logo_text_label = QtWidgets.QLabel()
-        self.logo_text_label.setText('Git Cola')
+        self.logo_text_label = qtutils.label(text='Git Cola')
         self.logo_text_label.setAlignment(Qt.AlignLeft | Qt.AlignCenter)
 
         font = self.logo_text_label.font()
@@ -429,7 +425,7 @@ def translators_text():
         dict(name='Vitor Lobo', title=N_('Brazilian translation')),
         dict(name='Zhang Han', title=N_('Simplified Chinese translation')),
         dict(name='Łukasz Wojniłowicz', title=N_('Polish translation')),
-        dict(name='Igor Kopach', title=N_('Ukranian translation')),
+        dict(name='Igor Kopach', title=N_('Ukrainian translation')),
         dict(name='Gyuris Gellért', title=N_('Hungarian translation')),
         dict(name='fu7mu4', title=N_('Japanese translation')),
         dict(name='Barış ÇELİK', title=N_('Turkish translation')),
@@ -508,6 +504,7 @@ def show_shortcuts():
         hotkeys_url = 'file://' + hotkeys_html
     if not os.path.isfile(hotkeys_html):
         hotkeys_url = 'https://git-cola.gitlab.io/share/doc/git-cola/hotkeys.html'
+        hotkeys_html = None
     try:
         from qtpy import QtWebEngineWidgets
     except (ImportError, qtpy.PythonQtError):
@@ -521,7 +518,14 @@ def show_shortcuts():
     widget.setWindowTitle(N_('Shortcuts'))
 
     web = QtWebEngineWidgets.QWebEngineView()
-    web.setUrl(QtCore.QUrl(hotkeys_url))
+    if hotkeys_html:
+        with open(hotkeys_html, encoding='utf-8') as hotkeys_file:
+            html = hotkeys_file.read()
+        if utils.is_darwin():
+            html = html.replace('Ctrl', 'Cmd')
+        web.setHtml(html)
+    else:
+        web.setUrl(QtCore.QUrl(hotkeys_url))
 
     layout = qtutils.hbox(defs.no_margin, defs.spacing, web)
     widget.setLayout(layout)
