@@ -44,9 +44,9 @@ class CommitMessageEditor(QtWidgets.QFrame):
         self.model = model = context.model
         self.spellcheck_initialized = False
         self.spellcheck = spellcheck.NorvigSpellCheck()
-        self.spellcheck.add_dictionaries(cfg.get_all('cola.dictionary'))
+        self.spellcheck.add_dictionaries(prefs.spelling_dictionaries(context))
         self.spellcheck.set_aspell_enabled(prefs.aspell_enabled(context))
-        self.spellcheck.set_aspell_langs(cfg.get_all('cola.aspell.lang'))
+        self.spellcheck.set_aspell_langs(prefs.aspell_languages(context))
 
         self._linebreak = None
         self._textwidth = None
@@ -296,6 +296,11 @@ class CommitMessageEditor(QtWidgets.QFrame):
         self.setFocusProxy(self.summary)
 
         cfg.user_config_changed.connect(self.config_changed)
+        self.context.notifier.ready.connect(self._ready, type=Qt.QueuedConnection)
+
+    def _ready(self):
+        """Called when the app is ready for events"""
+        self.context.runtask.run(self.spellcheck.init)
 
     def config_changed(self, key, value):
         if key != prefs.SPELL_CHECK:
