@@ -191,13 +191,15 @@ class DiffTextEdit(VimHintedPlainTextEdit):
     def __init__(
         self, context, parent, is_commit=False, whitespace=True, numbers=False
     ):
-        VimHintedPlainTextEdit.__init__(self, context, '', parent=parent)
+        super().__init__(context, '', parent=parent)
         # Diff/patch syntax highlighter
+        self.max_diff_size = 0
         self.highlighter = DiffSyntaxHighlighter(
             context, self.document(), is_commit=is_commit, whitespace=whitespace
         )
+        self.diff_lines = diffparse.DiffLines()
         if numbers:
-            self.numbers = DiffLineNumbers(context, self)
+            self.numbers = DiffLineNumbers(context, self, diff_lines=self.diff_lines)
             self.numbers.hide()
         else:
             self.numbers = None
@@ -214,13 +216,25 @@ class DiffTextEdit(VimHintedPlainTextEdit):
         self.menu_actions.append(self.copy_diff_action)
         self.cursorPositionChanged.connect(self._cursor_changed)
         self.selectionChanged.connect(self._selection_changed)
+        self.mouse_zoomed.connect(self.update_block_cursor)
 
     def setFont(self, font):
         """Override setFont() so that we can use a custom "block" cursor"""
         super().setFont(font)
-        if prefs.block_cursor(self.context):
-            width = qtutils.text_width(font, 'M')
-            self.setCursorWidth(width)
+        self.update_block_cursor(font=font)
+
+    def update_block_cursor(self, font=None):
+        """Update the block cusor width"""
+        self._update_block_cursor(self.context, font=font)
+
+    def _update_block_cursor(self, context, font=None):
+        """Update the block cusor width"""
+        if not prefs.block_cursor(context):
+            return
+        if font is None:
+            font = self.font()
+        width = qtutils.text_width(font, 'M')
+        self.setCursorWidth(width)
 
     def _cursor_changed(self):
         """Update the line number display when the cursor changes"""
@@ -256,21 +270,19 @@ class DiffTextEdit(VimHintedPlainTextEdit):
             scrollbar.setValue(scrollvalue)
         self.scrollvalue = None
 
-    def set_loading_message(self):
-        """Add a pending loading message in the diff view"""
-        self.hint.set_value('+++ ' + N_('Loading...'))
-        self.set_value('')
-
     def set_diff(self, diff):
-        """Set the diff text, but save the scrollbar"""
+        """Set the diff text and restore the scrollbar position post-update"""
         diff = diff.rstrip('\n')  # diffs include two empty newlines
+        diff = _truncate_diff(diff, self.max_diff_size)
+
         self.save_scrollbar()
 
-        self.hint.set_value('')
+        lines = self.diff_lines.parse(diff)
         if self.numbers:
-            self.numbers.set_diff(diff)
-        self.set_value(diff)
+            # The diff_lines parser is shared with self.numbers and updated above.
+            self.numbers.set_diff(diff, lines=lines)
 
+        self.set_value(diff)
         self.restore_scrollbar()
 
     def selected_diff_stripped(self):
@@ -339,12 +351,34 @@ def _strip_diff(value):
     return value
 
 
+def _truncate_diff(value, size):
+    """Truncate the diff to the specified number of megabytes"""
+    if size == 0:  # Unlimited
+        return value
+
+    # Technically size represents the number of unicode tokens not bytes, but it's good
+    # enough given that usually we're dealing with utf-8 text.
+    count = size * 1024 * 1024
+    if len(value) <= count:
+        return value
+
+    # Find the last newline starting from size so that the last line is a full, complete
+    # line rather than an invalid truncated invalid diff value.
+    newline = value.rfind('\n', 0, count)
+    if newline == -1:
+        return value[:count]
+
+    return value[:newline]
+
+
 class DiffLineNumbers(TextDecorator):
-    def __init__(self, context, parent):
+    """The diff viewer's line number display"""
+
+    def __init__(self, context, parent, diff_lines=None):
         TextDecorator.__init__(self, parent)
         self.highlight_line = -1
         self.lines = None
-        self.parser = diffparse.DiffLines()
+        self.parser = diff_lines or diffparse.DiffLines()
         self.formatter = diffparse.FormatDigits()
 
         font = qtutils.diff_font(context)
@@ -360,8 +394,11 @@ class DiffLineNumbers(TextDecorator):
         self._window = palette.color(QPalette.Window)
         self._disabled = palette.color(QPalette.Disabled, QPalette.Text)
 
-    def set_diff(self, diff):
-        self.lines = self.parser.parse(diff)
+    def set_diff(self, diff, lines=None):
+        """Update to a new diff display"""
+        if lines is None:
+            lines = self.parser.parse(diff)
+        self.lines = lines
         self.formatter.set_digits(self.parser.digits())
 
     def width_hint(self):
@@ -476,16 +513,25 @@ class Viewer(QtWidgets.QFrame):
         self.model = model = context.model
         self.images = []
         self.pixmaps = []
-        self.options = options = Options(self)
+        italic_font = self.font()
+        italic_font.setItalic(True)
+
         self.filename = PlainTextLabel(parent=self)
         self.filename.setAlignment(Qt.AlignVCenter | Qt.AlignLeft)
-        font = self.font()
-        font.setItalic(True)
-        self.filename.setFont(font)
+        self.filename.setFont(italic_font)
         self.filename.elide()
+        self.options = options = Options(self, filename=self.filename)
+
+        diffstat_font = self.font()
+        diffstat_font.setPointSize(diffstat_font.pointSize() - 1)
+        self.diffstat = PlainTextLabel(parent=self)
+        self.diffstat.setAlignment(Qt.AlignVCenter | Qt.AlignRight)
+        self.diffstat.setFont(diffstat_font)
+
         self.text = DiffEditor(context, options, self)
         self.image = imageview.ImageView(parent=self)
         self.image.setFocusPolicy(Qt.NoFocus)
+
         self.search_widget = TextSearchWidget(self.text, self)
         self.search_widget.hide()
         self._drag_has_patches = False
@@ -514,6 +560,13 @@ class Viewer(QtWidgets.QFrame):
         # Observe the file type
         model.file_type_changed.connect(self.set_file_type, type=Qt.QueuedConnection)
 
+        # Observe the diff text
+        model.diff_text_updated.connect(self.set_diff, type=Qt.QueuedConnection)
+        # Observe the diff loading state
+        self.context.notifier.listen(
+            cmds.Messages.DIFF_LOADING, self.set_loading_message
+        )
+
         # Observe the image mode combo box
         options.image_mode.currentIndexChanged.connect(lambda _: self.render())
         options.zoom_mode.currentIndexChanged.connect(lambda _: self.render())
@@ -525,6 +578,11 @@ class Viewer(QtWidgets.QFrame):
             hotkeys.SEARCH,
         )
 
+    def set_loading_message(self):
+        """Display an indicator that a diff is loading in the background"""
+        # The diffstat will be replaced with the real diffstat on load.
+        self.diffstat.set_text(N_('Loading...'))
+
     def dragEnterEvent(self, event):
         """Accepts drops if the mimedata contains patches"""
         super().dragEnterEvent(event)
@@ -572,6 +630,7 @@ class Viewer(QtWidgets.QFrame):
         state['image_diff_mode'] = self.options.image_mode.currentIndex()
         state['image_zoom_mode'] = self.options.zoom_mode.currentIndex()
         state['word_wrap'] = self.options.enable_word_wrapping.isChecked()
+        state['max_diff_size'] = self.options.max_diff_spinbox.value()
         return state
 
     def apply_state(self, state):
@@ -589,8 +648,23 @@ class Viewer(QtWidgets.QFrame):
 
         word_wrap = bool(state.get('word_wrap', True))
         self.set_word_wrapping(word_wrap, update=True)
+
+        max_diff_size = state.get('max_diff_size', 1)
+        self.text.max_diff_size = max_diff_size
+        self.options.max_diff_spinbox.set_value(max_diff_size)
         return True
 
+    def set_diff(self, diff):
+        """Update the diffstat display in reponse to the new diff"""
+        filename = self.context.selection.filename()
+        if filename:
+            removals = self.text.diff_lines.removals.count
+            additions = self.text.diff_lines.additions.count
+            diffstat = f'-{removals}  +{additions}'
+        else:
+            diffstat = ''
+        self.diffstat.set_text(diffstat)
+
     def set_diff_type(self, diff_type):
         """Manage the image and text diff views when selection changes"""
         # The "diff type" is whether the diff viewer is displaying an image.
@@ -627,6 +701,7 @@ class Viewer(QtWidgets.QFrame):
     def set_show_filenames(self, enabled, update=False):
         """Enable/disable displaying the selected filename"""
         self.filename.setVisible(enabled)
+        self.diffstat.setVisible(enabled)
         if update:
             with qtutils.BlockSignals(self.options.show_filenames):
                 self.options.show_filenames.setChecked(enabled)
@@ -771,10 +846,11 @@ class Options(QtWidgets.QWidget):
     XOR = 2
     PIXEL_XOR = 3
 
-    def __init__(self, parent):
+    def __init__(self, parent, filename=None):
         super().__init__(parent)
         # Create widgets
         self.widget = parent
+        self.filename = filename  # The filename plain text display.
         self.ignore_space_at_eol = self.add_option(
             N_('Ignore changes in whitespace at EOL')
         )
@@ -792,6 +868,29 @@ class Options(QtWidgets.QWidget):
         self.enable_word_wrapping = qtutils.add_action_bool(
             self, N_('Enable word wrapping'), self.set_word_wrapping, True
         )
+        self.max_diff_label = QtWidgets.QLabel(
+            N_('Maximum diff size in megabytes (MB)'), self
+        )
+        self.max_diff_spinbox = standard.SpinBox(
+            value=1,
+            mini=0,
+            maxi=9999,
+            suffix='\tMB',
+            tooltip=N_('The maximum diff size in megabytes (MB)'),
+            parent=self,
+        )
+        self.max_diff_spinbox.setSpecialValueText(N_('Unlimited'))
+        self.max_diff_widget = QtWidgets.QWidget(self)
+        self.max_diff_layout = qtutils.hbox(
+            defs.no_margin,
+            defs.button_spacing,
+            self.max_diff_label,
+            qtutils.STRETCH,
+            self.max_diff_spinbox,
+        )
+        self.max_diff_widget.setLayout(self.max_diff_layout)
+        self.max_diff_action = QtWidgets.QWidgetAction(self)
+        self.max_diff_action.setDefaultWidget(self.max_diff_widget)
 
         self.options = qtutils.create_toolbutton(
             tooltip=N_('Diff Options'), icon=icons.configure()
@@ -820,6 +919,8 @@ class Options(QtWidgets.QWidget):
 
         self.menu = menu = qtutils.create_menu(N_('Diff Options'), self.options)
         self.options.setMenu(menu)
+        menu.addAction(self.max_diff_action)
+        menu.addSeparator()
         menu.addAction(self.ignore_space_at_eol)
         menu.addAction(self.ignore_space_change)
         menu.addAction(self.ignore_all_space)
@@ -834,10 +935,12 @@ class Options(QtWidgets.QWidget):
         layout = qtutils.hbox(
             defs.no_margin,
             defs.button_spacing,
-            self.image_mode,
-            self.zoom_mode,
             self.options,
             self.toggle_image_diff,
+            self.filename,
+            self.image_mode,
+            self.zoom_mode,
+            qtutils.STRETCH,
         )
         self.setLayout(layout)
 
@@ -855,9 +958,7 @@ class Options(QtWidgets.QWidget):
 
     def set_diff_type(self, diff_type):
         """Toggle between image and text diffs"""
-        is_text = diff_type == main.Types.TEXT
         is_image = diff_type == main.Types.IMAGE
-        self.options.setVisible(is_text)
         self.image_mode.setVisible(is_image)
         self.zoom_mode.setVisible(is_image)
         if is_image:
@@ -891,7 +992,7 @@ class Options(QtWidgets.QWidget):
         self.widget.set_word_wrapping(value, update=False)
 
     def hide_advanced_options(self):
-        """Hide advanced options that are not applicable to the DiffWidget"""
+        """Hide advanced options that are not applicable to the CommitDiffWidget"""
         self.show_filenames.setVisible(False)
         self.show_line_numbers.setVisible(False)
         self.ignore_space_at_eol.setVisible(False)
@@ -916,27 +1017,39 @@ class DiffEditor(DiffTextEdit):
 
         self.action_apply_selection = qtutils.add_action(
             self,
-            'Apply',
+            N_('Apply'),
             self.apply_selection,
             hotkeys.STAGE_DIFF,
             hotkeys.STAGE_DIFF_ALT,
         )
 
-        self.action_revert_selection = qtutils.add_action(
-            self, 'Revert', self.revert_selection, hotkeys.REVERT, hotkeys.REVERT_ALT
+        self.action_revert_selection = qtutils.add_action_with_icon(
+            self,
+            icons.undo(),
+            N_('Revert Selected Lines'),
+            self.revert_selection,
+            hotkeys.REVERT,
+            hotkeys.REVERT_ALT,
+        )
+
+        self.action_revert_unstaged_edits = qtutils.add_action_with_icon(
+            self,
+            icons.undo(),
+            cmds.RevertUnstagedEdits.name(),
+            cmds.run(cmds.RevertUnstagedEdits, self.context),
+            hotkeys.REVERT_UNSTAGED_EDITS,
         )
-        self.action_revert_selection.setIcon(icons.undo())
 
         self.action_edit_and_apply_selection = qtutils.add_action(
             self,
-            'Edit and Apply',
+            N_('Edit and Apply'),
             partial(self.apply_selection, edit=True),
             hotkeys.EDIT_AND_STAGE_DIFF,
         )
 
         self.action_edit_and_revert_selection = qtutils.add_action(
             self,
-            'Edit and Revert',
+            N_('Edit and Revert'),
             partial(self.revert_selection, edit=True),
             hotkeys.EDIT_AND_REVERT,
         )
@@ -952,19 +1065,25 @@ class DiffEditor(DiffTextEdit):
         self.move_down = actions.move_down(self)
 
         model.diff_text_updated.connect(self.set_diff, type=Qt.QueuedConnection)
+        model.mode_changed.connect(self.update_actions, type=Qt.QueuedConnection)
 
         selection_model.selection_changed.connect(
-            self.refresh, type=Qt.QueuedConnection
+            self.update_actions, type=Qt.QueuedConnection
         )
         # Update the selection model when the cursor changes
         self.cursorPositionChanged.connect(self._update_line_number)
 
         qtutils.connect_button(options.toggle_image_diff, self.toggle_diff_type)
+        self.options.max_diff_spinbox.valueChanged.connect(self.set_max_diff_size)
+
+    def set_max_diff_size(self, value):
+        """Set the max diff state on the diff widget"""
+        self.max_diff_size = value
 
     def toggle_diff_type(self):
         cmds.do(cmds.ToggleDiffType, self.context)
 
-    def refresh(self):
+    def update_actions(self):
         enabled = False
         s = self.selection_model.selection()
         model = self.model
@@ -1059,10 +1178,10 @@ class DiffEditor(DiffTextEdit):
 
                 self.action_revert_selection.setText(revert_text)
                 add_action(self.action_revert_selection)
-
                 stage_action_added = self._add_stage_or_unstage_action(
                     menu, add_action, stage_action_added
                 )
+                add_action(self.action_revert_unstaged_edits)
                 # Do not show the "edit" action when the file does not exist.
                 add_action(qtutils.menu_separator(menu))
                 if filename and core.exists(filename):
@@ -1507,7 +1626,7 @@ class AuthorLabel(PlainTextLabel):
         super().context_menu_actions(menu)
 
 
-class DiffWidget(QtWidgets.QWidget):
+class CommitDiffWidget(QtWidgets.QWidget):
     """Display commit metadata and text diffs"""
 
     def __init__(self, context, parent, is_commit=False, options=None):
@@ -1575,7 +1694,7 @@ class DiffWidget(QtWidgets.QWidget):
     def start_diff_task(self, task):
         """Clear the display and start a diff-gathering task"""
         self.diff.save_scrollbar()
-        self.diff.set_loading_message()
+        cmds.do(cmds.DiffLoading, self.context)
         self.context.runtask.start(task, result=self.set_diff)
 
     def set_diff_oid(self, oid, filename=None):
@@ -1780,7 +1899,7 @@ class ApplyPatches(standard.Dialog):
         self.tree.setHeaderHidden(True)
         self.tree.itemSelectionChanged.connect(self._tree_selection_changed)
 
-        self.diffwidget = DiffWidget(context, self, is_commit=True)
+        self.diffwidget = CommitDiffWidget(context, self, is_commit=True)
 
         self.add_button = qtutils.create_toolbutton(
             text=N_('Add'), icon=icons.add(), tooltip=N_('Add patches (+)')
